package procon;

// Definition of class HoldIntegerSynchronized that
// uses thread synchronisation to ensure that both
// threads access sharedInt at the proper times.
import java.text.DecimalFormat;
import javax.swing.*;

public class HoldIntegerSynchronized {

    // Array of shared locations
    private int sharedInt[] = {-1, -1, -1, -1, -1};
    // Variables to maintain buffer information
    private boolean writeable = true;
    private boolean readable = false;
    private int readLocation = 0, writeLocation = 0;
    // GUI component to display output
    private JTextArea outputArea;

    // Initialise HoldIntegerSynchronized
    public HoldIntegerSynchronized(JTextArea output) {
        outputArea = output;
    }

    // Synchronised method allows only one thread at a time to
    // invoke this method to set a value in a particular
    // HoldIntegerSynchronized object
    public synchronized void setSharedInt(int value) {
        while (!writeable) {

            // Thread that called this method must wait
            try {

                // Update Swing GUI component
                SwingUtilities.invokeLater(new UpdateThread(
                        outputArea, " WAITING TO PRODUCE " + value));

                wait();
            } // Process InterrupteException while thread waiting
            catch (InterruptedException exception) {
                System.err.println(exception.toString());
            }
        }
        // Place value in writeLocation
        sharedInt[writeLocation] = value;

        // Indicate that consumer can read a value
        readable = true;

        // Update Swing GUI component
        SwingUtilities.invokeLater(new UpdateThread(outputArea,
                "\nProduced " + value + " into cell "
                + writeLocation));

        // Update writeLocation for future write operation
        writeLocation = (writeLocation + 1) % 5;

        // Update Swing GUI component
        SwingUtilities.invokeLater(new UpdateThread(outputArea,
                "\twrite " + writeLocation + "\tread "
                + readLocation));

        displayBuffer(outputArea, sharedInt);

        // Test if buffer is full
        if (writeLocation == readLocation) {
            writeable = false;

            // Update Swing GUI component
            SwingUtilities.invokeLater(new UpdateThread(outputArea,
                    "\nBUFFER FULL"));
        }

        // Tell a waiting thread to become ready
        notify();

    } // End method setSharedInt

    // Synchronised method allows only one thread at a time to
    // invoke this method to get a value from a particular
    // HoldIntegerSynchronized object
    public synchronized int getSharedInt() {
        int value;

        while (!readable) {

            // Thread that called this method must wait
            try {

                // Update Swing GUI component
                SwingUtilities.invokeLater(new UpdateThread(
                        outputArea, " WAITING TO CONSUME"));
                wait();
            } // Process InterrupteException while thread waiting
            catch (InterruptedException exception) {
                System.err.println(exception.toString());
            }
        }

        // Indicate that producer can write a value
        writeable = true;

        // Obtain value at current readLocation
        value = sharedInt[readLocation];

        // Update Swing GUI component
        SwingUtilities.invokeLater(new UpdateThread(outputArea,
                "\nConsumed " + value + " from cell "
                + readLocation));

        // Update read location for future read operation
        readLocation = (readLocation + 1) % 5;

        // Update Swing GUI component
        SwingUtilities.invokeLater(new UpdateThread(outputArea,
                "\twrite " + writeLocation + "\tread "
                + readLocation));

        displayBuffer(outputArea, sharedInt);

        // Test if buffer is empty
        if (readLocation == writeLocation) {
            readable = false;

            // Update Swing GUI component
            SwingUtilities.invokeLater(new UpdateThread(
                    outputArea, "\nBUFFER EMPTY"));
        }

        // Tell a waiting thread to become ready
        notify();

        return value;

    } // End method getSharedInt

    // Display contents of shared buffer
    public void displayBuffer(JTextArea outputArea,
            int buffer[]) {
        DecimalFormat formatNumber = new DecimalFormat(" #;-#");
        StringBuffer outputBuffer = new StringBuffer();

        // Place buffer elements in outputBuffer
        for (int count = 0; count < buffer.length; count++) {
            outputBuffer.append(
                    " " + formatNumber.format(buffer[ count]));
        }

        // Update Swing GUI component
        SwingUtilities.invokeLater(new UpdateThread(outputArea,
                "\tbuffer: " + outputBuffer));
    }
}
